@@ -1,3 +1,5 @@
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 from prewikka import version
 from prewikka.database import SQLScript
 
@@ -40,8 +42,31 @@ CREATE TABLE Prewikka_User_Configuration (
     userid VARCHAR(255) NOT NULL,
     view VARCHAR(32) NULL,
     name VARCHAR(255) NOT NULL,
-    value VARCHAR(255) NULL
+    value VARCHAR(255) NULL,
+    PRIMARY KEY(userid, view, name)
 ) ENGINE=InnoDB;
 
-CREATE INDEX prewikka_user_configuration_index ON Prewikka_User_Configuration (name, userid, view);
+DROP TABLE IF EXISTS Prewikka_Crontab;
+CREATE TABLE Prewikka_Crontab (
+    id BIGINT UNSIGNED NOT NULL PRIMARY KEY AUTO_INCREMENT,
+    name VARCHAR(255) NOT NULL,
+    userid VARCHAR(32) NULL,
+    schedule VARCHAR(32) NULL,
+    ext_type VARCHAR(255) NULL,
+    ext_id INTEGER NULL,
+    base DATETIME NOT NULL,
+    enabled TINYINT DEFAULT 1,
+    runcnt  INTEGER DEFAULT 0,
+    error TEXT NULL
+) ENGINE=InnoDB;
+
+DROP TABLE IF EXISTS Prewikka_History_Query;
+CREATE TABLE Prewikka_History_Query (
+    userid VARCHAR(32) NOT NULL,
+    formid VARCHAR(255) NOT NULL,
+    query TEXT NOT NULL,
+    query_hash VARCHAR(32) NOT NULL,
+    timestamp DATETIME NOT NULL,
+    PRIMARY KEY(userid, formid, query_hash)
+) ENGINE=InnoDB;
 """)
