@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2016 CS-SI. All Rights Reserved.
+# Copyright (C) 2014-2017 CS-SI. All Rights Reserved.
 # Author: Antoine Luong <antoine.luong@c-s.fr>
 #
 # This file is part of the Prewikka program.
@@ -18,7 +18,13 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 
-import os, glob, pkg_resources
+from __future__ import absolute_import, division, print_function, unicode_literals
+
+import glob
+import os
+
+import pkg_resources
+
 
 def getThemes():
     theme_list = []
