@@ -1,7 +1,8 @@
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 import pkg_resources
+from prewikka import version, view
 
-from . import templates
-from prewikka import view, version
 
 class About(view.View):
     plugin_name = "About"
@@ -12,5 +13,5 @@ class About(view.View):
     plugin_description = N_("Prelude About page")
     plugin_htdocs = (("about", pkg_resources.resource_filename(__name__, 'htdocs')),)
 
-    view_template = templates.About
-    view_parameters = view.Parameters
+    view_template = pkg_resources.resource_filename(__name__, 'templates/about.mak')
+    view_help = "#about"
