@@ -1,6 +1,9 @@
-from messagesummary import AlertSummary, HeartbeatSummary
+from __future__ import absolute_import, division, print_function, unicode_literals
 from prewikka import pluginmanager, version
 
+from .messagesummary import AlertSummary, HeartbeatSummary
+
+
 class MessageSummary(pluginmanager.PluginPreload):
     plugin_name = "Detailed alert and heartbeat"
     plugin_author = version.__author__
