@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2016 CS-SI. All Rights Reserved.
+# Copyright (C) 2004-2017 CS-SI. All Rights Reserved.
 # Author: Yoann Vandoorselaere <yoannv@gmail.com>
 #
 # This file is part of the Prewikka program.
@@ -17,31 +17,27 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 import hashlib
 import re
-import time
+import string
 import struct
-import socket
 import urllib
-import pkg_resources
-
 from datetime import datetime
-from prewikka import view, usergroup, utils, resolve, localization, env, hookmanager
 
-from . import templates
+import pkg_resources
+from prewikka import hookmanager, localization, resolve, resource, template, utils, view, mainmenu
+from prewikka.dataprovider import Criterion
+from prewikka.utils import html
 
-def getUriCriteria(parameters):
-    if not parameters.has_key("messageid"):
-        return None
 
-    if parameters.has_key("analyzerid"):
-        criteria = "alert.analyzer(-1).analyzerid = '%s' &&" % utils.escape_criteria(parameters["analyzerid"])
-    else:
-        criteria = ""
+def getUriCriteria(ptype, analyzerid, messageid):
+    criteria = Criterion()
+    if analyzerid:
+        criteria += Criterion("%s.analyzer.analyzerid" % (ptype), "=", analyzerid)
 
-    criteria += "alert.messageid = '%s'" % utils.escape_criteria(parameters["messageid"])
-
-    return criteria
+    return criteria + Criterion("%s.messageid" % (ptype), "=", messageid)
 
 
 class Table(object):
@@ -73,7 +69,7 @@ class Table(object):
             return
 
         if not parent:
-            self.dataset["sections"].append(self._current_section)
+            env.request.dataset["sections"].append(self._current_section)
             self._current_section = None
         else:
             parent["sections"].append(self._current_section)
@@ -87,11 +83,11 @@ class Table(object):
                                                   "value": value,
                                                   "emphase": emphase })
 
-    def beginTable(self, cl="message_summary", style="", odd_even=False):
+    def beginTable(self, cl="table-striped table-bordered", style="", odd_even=False):
         table = {}
         table["rows"] = []
         table["odd_even"] = odd_even
-        table["class"] = cl
+        table["class"] = "table " + cl
         table["style"] = style
         table["parent"] = self._current_table or self._current_section
         self._current_table = table
@@ -102,7 +98,7 @@ class Table(object):
         has_data = False
 
         if len(self._current_table["rows"]) <= 1 :
-            if not parent or not parent.has_key("rows"):
+            if not parent or not "rows" in parent:
                 self._current_table = None
             else:
                 self._current_table = parent
@@ -112,7 +108,7 @@ class Table(object):
             self._current_section["tables"].append(self._current_table)
             self._current_table = None
         else:
-            if parent.has_key("rows"):
+            if "rows" in parent:
                 col = { "name": None, "header": None, "emphase": None, "tables": [ self._current_table ] }
                 if len(parent["rows"]):
                     parent["rows"][-1] += [col]
@@ -172,7 +168,7 @@ class HeaderTable(Table):
 
         for field in self.field_list:
 
-            if not dataset.has_key(field[0]) and not field[2]:
+            if not field[0] in dataset and not field[2]:
                 continue
 
             if field[2]:
@@ -203,44 +199,35 @@ class TcpIpOptions(Table):
         if (bits & flag) >> shift:
             return "X"
         else:
-            return "&nbsp;"
+            return resource.HTMLSource("&nbsp;")
 
     def _decodeOption8(self, data):
-        return str(struct.unpack(">B", data)[0])
+        return text_type(struct.unpack(b">B", data)[0])
 
     def _decodeOption16(self, data):
-        return str(struct.unpack(">H", data)[0])
+        return text_type(struct.unpack(b">H", data)[0])
 
     def _decodeOption32(self, data):
-        return str(struct.unpack(">L", data)[0])
+        return text_type(struct.unpack(b">L", data)[0])
 
     def _decodeOptionTimestamp(self, data):
-        x = struct.unpack(">LL", data)
-        return "TS Value (%d)<br/>TS Echo Reply (%d)" % (x[0], x[1])
+        x = struct.unpack(b">LL", data)
+        return resource.HTMLSource("TS Value (%d)<br/>TS Echo Reply (%d)") % (x[0], x[1])
 
     def _decodeOptionSack(self, data):
-        x = struct.unpack(">" + "L" * (len(data) / 4), data)
-
-        s = ""
-        for i in x:
-            if len(s):
-                s += "<br/>"
-
-            s += str(i)
-
-        return s
-
+        x = struct.unpack(b">" + "L" * (len(data) / 4), data)
+        return resource.HTMLSource("<br/>").join(text_type(i) for i in x)
 
     def _decodeOptionMd5(self, data):
-        md = hashlib.md5(struct.unpack(">B" * 16, data)[0])
+        md = hashlib.md5(struct.unpack(b">B" * 16, data)[0])
         return md.hexdigest()
 
     def _decodeOptionPartialOrderProfile(self, data):
-        x = struct.unpack(">B", data)
+        x = struct.unpack(b">B", data)
         return "Start_Flags=%d End_Flags=%d" % (data & 0x80, data & 0x40)
 
     def _decodeOptionTcpAltChecksumRequest(self, data):
-        x = struct.unpack(">B", data)
+        x = struct.unpack(b">B", data)
         if x == 0:
             return "TCP checksum"
 
@@ -317,14 +304,14 @@ class TcpIpOptions(Table):
             self.newTableCol(idx, option[0])
 
             if len(dec) == 2 and dec[1] != -1 and dec[1] != option[1]:
-                self.newTableCol(idx, "<b style='color:red;'>%d</b> (expected %d)" % (option[1], dec[1]))
+                self.newTableCol(idx, resource.HTMLSource("<b style='color:red;'>%d</b> (expected %d)" % (option[1], dec[1])))
             else:
                 self.newTableCol(idx, "%d" % option[1])
 
             if len(dec) == 3 and (dec[1] == -1 or dec[1] == option[1]):
                 self.newTableCol(idx, "%s" % dec[2](option[2]))
             else:
-                self.newTableCol(idx, "&nbsp;")
+                self.newTableCol(idx, resource.HTMLSource("&nbsp;"))
 
         self.endTable()
 
@@ -349,18 +336,19 @@ class TcpIpOptions(Table):
 
 
 
-class MessageParameters(view.Parameters):
+class MessageParameters(mainmenu.MainMenuParameters):
     def register(self):
+        mainmenu.MainMenuParameters.register(self)
         view.Parameters.register(self)
-        self.optional("ident", long)
-        self.optional("analyzerid", str)
-        self.optional("messageid", str)
+        self.optional("analyzerid", text_type)
+        self.optional("messageid", text_type)
 
 
 class MessageSummary(Table, view.View):
     view_parameters = MessageParameters
     view_permissions = [ N_("IDMEF_VIEW") ]
-    view_template = templates.MessageSummary
+    view_template = template.PrewikkaTemplate(__name__, 'templates/messagesummary.mak')
+    view_extensions = (("menu", mainmenu.MainMenu),)
     plugin_htdocs = (("messagesummary", pkg_resources.resource_filename(__name__, 'htdocs')),)
 
     def __init__(self, *args, **kwargs):
@@ -381,7 +369,7 @@ class MessageSummary(Table, view.View):
             else:
                 return name
 
-        return '<a target="%s" href="%s">%s</a>' % (env.external_link_target, url, name)
+        return resource.HTMLSource('<a target="%s" href="%s">%s</a>' % (env.external_link_target, html.escape(url), html.escape(name)))
 
     def getTime(self, dt):
         if not dt:
@@ -425,7 +413,7 @@ class MessageSummary(Table, view.View):
 
         self.newTableEntry(_("Node location"), node["location"])
 
-        addr_list = None
+        addr_list = []
         node_name = None
         for addr in node["address"]:
             address = addr["address"]
@@ -434,15 +422,10 @@ class MessageSummary(Table, view.View):
 
             node_name = resolve.AddressResolve(address)
 
-            if addr_list:
-                addr_list += "<br/>"
-            else:
-                addr_list = ""
-
             if addr["category"] in ("ipv4-addr", "ipv6-addr", "ipv4-net", "ipv6-net") and env.enable_details:
-                addr_list += self.getUrlLink(address, "%s?host=%s" %(env.host_details_url, address))
+                addr_list.append(self.getUrlLink(address, "%s?host=%s" % (env.host_details_url, address)))
             else:
-                addr_list += address
+                addr_list.append(address)
 
         if node["name"]:
             self.newTableEntry(_("Node name"), node["name"])
@@ -450,10 +433,11 @@ class MessageSummary(Table, view.View):
         elif node_name is not None and node_name.resolveSucceed():
             self.newTableEntry(_("Node name (resolved)"), node_name)
 
-        self.newTableEntry(_("Node address"), addr_list)
+        if addr_list:
+            self.newTableEntry(_("Node address"), resource.HTMLSource("<br/>").join(addr_list))
 
     def buildAnalyzer(self, analyzer):
-        self.beginTable(cl="message_summary_no_border")
+        self.beginTable(cl="table-borderless")
 
         self.beginTable()
         self.newTableEntry(_("Model"), analyzer["model"], cl="section_alert_entry_value_emphasis")
@@ -490,7 +474,7 @@ class MessageSummary(Table, view.View):
 
         self.beginSection(_("Analyzer Path (%d not shown)") % len(l), display="none")
 
-        self.beginTable(cl="message_summary_no_border")
+        self.beginTable(cl="table-borderless")
 
         i = 1
         index = len(l) - 1
@@ -544,19 +528,16 @@ class MessageSummary(Table, view.View):
                     ignored[meaning] = value
                     break
 
-            links = []
-            for url, text in hookmanager.trigger("HOOK_ALERTSUMMARY_MEANING_LINK", alert, meaning, value):
-                if url:
-                    links.append("<a target='%s' href='%s'>%s</a>" % \
-                                 (env.external_link_target, url, text))
+            links = resource.HTMLSource()
+            for obj in filter(None, hookmanager.trigger("HOOK_ALERTSUMMARY_MEANING_LINK", alert, meaning, value)):
+                links += obj.to_string()
 
             if links:
-                meaning = "<a class='popup_menu_toggle'>%s</a><span class='popup_menu'>%s</span>" % \
-                          (meaning, "".join(links))
+                meaning = resource.HTMLSource("<a class='popup_menu_toggle'>%s</a><span class='popup_menu'>%s</span>") % (meaning, links)
 
-            if not ignored.has_key(meaning):
-                self.newTableCol(index, meaning or "Data content")
-                self.newTableCol(index, utils.escape_html_string(value) if value is not None else None)
+            if not meaning in ignored:
+                self.newTableCol(index, resource.HTMLSource(meaning or "Data content"))
+                self.newTableCol(index, html.escape(value) if value else None)
                 index += 1
 
         self.endTable()
@@ -569,9 +550,9 @@ class MessageSummary(Table, view.View):
         ip.register(_("TOS"), "ip_tos")
         ip.register(_("Length"), "ip_len")
         ip.register(_("Id"), "ip_id")
-        ip.register("R<br/>F", "ip_off", self._isFlagSet, (0x8000, 15))
-        ip.register("D<br/>F", "ip_off", self._isFlagSet, (0x4000, 14))
-        ip.register("M<br/>F", "ip_off", self._isFlagSet, (0x2000, 13))
+        ip.register(resource.HTMLSource("R<br/>F"), "ip_off", self._isFlagSet, (0x8000, 15))
+        ip.register(resource.HTMLSource("D<br/>F"), "ip_off", self._isFlagSet, (0x4000, 14))
+        ip.register(resource.HTMLSource("M<br/>F"), "ip_off", self._isFlagSet, (0x2000, 13))
         ip.register(_("Ip offset"), "ip_off", (lambda x: x & 0x1fff))
         ip.register(_("TTL"), "ip_ttl")
         ip.register(_("Protocol"), "ip_proto")
@@ -588,14 +569,14 @@ class MessageSummary(Table, view.View):
         tcp.register("Ack #", "tcp_ack")
         tcp.register(_("Header length"), "tcp_off")
         tcp.register(_("Reserved"), "tcp_res")
-        tcp.register("R<br/>1", "tcp_flags", self._isFlagSet, (0x80,))
-        tcp.register("R<br/>2", "tcp_flags", self._isFlagSet, (0x40,))
-        tcp.register("U<br/>R<br/>G", "tcp_flags", self._isFlagSet, (0x20,))
-        tcp.register("A<br/>C<br/>K", "tcp_flags", self._isFlagSet, (0x10,))
-        tcp.register("P<br/>S<br/>H", "tcp_flags", self._isFlagSet, (0x08,))
-        tcp.register("R<br/>S<br/>T", "tcp_flags", self._isFlagSet, (0x04,))
-        tcp.register("S<br/>Y<br/>N", "tcp_flags", self._isFlagSet, (0x02,))
-        tcp.register("F<br/>I<br/>N", "tcp_flags", self._isFlagSet, (0x01,))
+        tcp.register(resource.HTMLSource("R<br/>1"), "tcp_flags", self._isFlagSet, (0x80,))
+        tcp.register(resource.HTMLSource("R<br/>2"), "tcp_flags", self._isFlagSet, (0x40,))
+        tcp.register(resource.HTMLSource("U<br/>R<br/>G"), "tcp_flags", self._isFlagSet, (0x20,))
+        tcp.register(resource.HTMLSource("A<br/>C<br/>K"), "tcp_flags", self._isFlagSet, (0x10,))
+        tcp.register(resource.HTMLSource("P<br/>S<br/>H"), "tcp_flags", self._isFlagSet, (0x08,))
+        tcp.register(resource.HTMLSource("R<br/>S<br/>T"), "tcp_flags", self._isFlagSet, (0x04,))
+        tcp.register(resource.HTMLSource("S<br/>Y<br/>N"), "tcp_flags", self._isFlagSet, (0x02,))
+        tcp.register(resource.HTMLSource("F<br/>I<br/>N"), "tcp_flags", self._isFlagSet, (0x01,))
         tcp.register(_("Window"), "tcp_win")
         tcp.register(_("Checksum"), "tcp_sum")
         tcp.register(_("URP"), "tcp_urp")
@@ -630,14 +611,22 @@ class MessageSummary(Table, view.View):
     def buildPayloadTable(self, alert):
         data = HeaderTable()
         data.register(_("Payload"), "payload")
-        #data.register("ASCII Payload", "payload", utils.escape_html_string)
+        #data.register("ASCII Payload", "payload", html.escape)
         return data
 
+    def render(self):
+        self._current_table = None
+        self._current_section = None
+
 
 class AlertSummary(TcpIpOptions, MessageSummary):
     def __init__(self):
         MessageSummary.__init__(self)
 
+    @staticmethod
+    def _get_alert_ident_criterion(analyzerid, ident):
+        return Criterion("alert.analyzer.analyzerid", "=", analyzerid) & Criterion("alert.messageid", "=", ident)
+
     def buildAlertIdent(self, alert, parent):
         calist = { }
 
@@ -654,38 +643,35 @@ class AlertSummary(TcpIpOptions, MessageSummary):
                         analyzerid = a["analyzerid"]
                         break
 
-            if not calist.has_key(analyzerid):
-                calist[analyzerid] = []
+            calist.setdefault(analyzerid, []).append(alertident["alertident"])
 
-            calist[analyzerid].append(alertident["alertident"])
-
-        idx = 1
-        for analyzerid in calist.keys():
+        for idx, analyzerid in enumerate(calist):
 
             content = ""
-            missing = 0
+            criteria = Criterion()
             for ident in calist[analyzerid]:
-                criteria = "alert.analyzer.analyzerid = '%s' && alert.messageid = '%s'" % (analyzerid, ident)
+                criteria |= self._get_alert_ident_criterion(analyzerid, ident)
 
-                results = env.idmef_db.getAlertIdents(criteria)
-                if len(results) == 0:
-                    missing += 1
-                    #content += "<li>" + _("Invalid 'analyzerid:messageid' pair, '%(analyzerid):%(messageid)'") % { "analyzerid": analyzerid, "messageid": ident } + "</li>"
-                else:
-                    alert = env.idmef_db.getAlert(results[0], htmlsafe=True)["alert"]
-                    link = utils.create_link("/".join(env.request.web.path_elements[:2] + [self.view_id]), {"ident": results[0]})
-                    content += '<li><a class="widget-link" title="%s" href="%s">%s</a></li>' % (_("Alert details"), link, alert["classification.text"])
+            results = env.dataprovider.query(["alert.messageid", "alert.classification.text"], criteria)
 
+            for ident, classif in results:
+                link = url_for(".", analyzerid=analyzerid, messageid=ident)
+                content += '<li><a title="%s" href="%s">%s</a></li>' % (_("Alert details"), link, html.escape(classif))
+
+            missing = len(calist[analyzerid]) - len(results)
             if missing > 0:
                 content += "<li>" + (_("%d linked alerts missing (probably deleted)") % missing) + "</li>"
 
-            self.newTableCol(idx, "<ul style='padding: 0px; margin: 0px 0px 0px 10px;'>%s</ul>" % content)
-            self.buildAnalyzer(alert["analyzer(-1)"])
+            self.newTableCol(idx + 1, resource.HTMLSource("<ul style='padding: 0px; margin: 0px 0px 0px 10px;'>%s</ul>" % content))
+
+            linked_alerts = env.dataprovider.get(self._get_alert_ident_criterion(analyzerid, ident))
+            if linked_alerts:
+                self.buildAnalyzer(linked_alerts[0]["alert.analyzer(-1)"])
+            else:
+                self.newTableCol(1, None)
+
             self.newTableRow()
 
-            idx += 1
-
-
     def buildCorrelationAlert(self, alert):
         ca = alert.get("correlation_alert")
         if not ca:
@@ -781,7 +767,7 @@ class AlertSummary(TcpIpOptions, MessageSummary):
             user_str += "(%d)" % number
 
         elif number:
-            user_str = str(number)
+            user_str = text_type(number)
 
         if tty:
             user_str += " on tty " + tty
@@ -912,7 +898,7 @@ class AlertSummary(TcpIpOptions, MessageSummary):
             return
 
         if service["port"]:
-            port = str(service["port"])
+            port = text_type(service["port"])
             if env.enable_details:
                 self.newTableEntry(_("Port"), self.getUrlLink(port, "%s?port=%s" % (env.port_details_url, port)))
             else:
@@ -1021,15 +1007,14 @@ class AlertSummary(TcpIpOptions, MessageSummary):
 
         return section
 
-    def render(self):
-        criteria = getUriCriteria(self.parameters)
-        if criteria is not None:
-            ident = env.idmef_db.getAlertIdents(criteria)[0]
-        else:
-            ident = self.parameters["ident"]
+    @view.route("/alerts/summary/<analyzerid>:<messageid>")
+    @view.route("/alerts/summary/<messageid>")
+    def render(self, analyzerid=None, messageid=None):
+        MessageSummary.render(self)
 
-        alert = env.idmef_db.getAlert(ident, htmlsafe=True)["alert"]
-        self.dataset["sections"] = [ ]
+        alert = env.dataprovider.get(getUriCriteria("alert", analyzerid, messageid))[0]["alert"]
+
+        env.request.dataset["sections"] = [ ]
 
         self.beginSection(self.getSectionName(alert))
 
@@ -1085,7 +1070,7 @@ class AlertSummary(TcpIpOptions, MessageSummary):
 
             self.beginSection(_("Network centric information"))
 
-            self.beginTable(cl="message_summary_no_border")
+            self.beginTable(cl="table-borderless")
             ip.render_table(self, "IP", ignored_value)
             self.ipOptionRender(ip_options)
 
@@ -1095,14 +1080,16 @@ class AlertSummary(TcpIpOptions, MessageSummary):
             udp.render_table(self, "UDP", ignored_value)
             icmp.render_table(self, "ICMP", ignored_value)
 
-            if ignored_value.has_key("payload"):
+            if "payload" in ignored_value:
                 val = {}
 
-                payload = utils.escape_html_string(utils.hexdump(ignored_value["payload"])).replace(" ", "&nbsp;")
-                val["payload"] = "<span class='fixed'>%s</span>" % payload
+                payload = html.escape(utils.hexdump(ignored_value["payload"])).replace(" ", resource.HTMLSource("&nbsp;"))
+                val["payload"] = resource.HTMLSource("<span class='fixed'>%s</span>" % payload)
                 data.render_table(self, _("Payload"), val)
 
-                val["payload"] = "<div style='overflow: auto;'>%s</div>" % utils.escape_html_string(ignored_value["payload"]).replace("\n", "<br/>")
+                pset = set(string.printable)
+                payload = ''.join((i if i in pset else '.' for i in ignored_value["payload"]))
+                val["payload"] = resource.HTMLSource("<div style='overflow: auto;'>%s</div>" % html.escape(payload).replace("\n", resource.HTMLSource("<br/>")))
                 data.render_table(self, _("ASCII Payload"), val)
 
             self.endTable()
@@ -1110,15 +1097,14 @@ class AlertSummary(TcpIpOptions, MessageSummary):
 
 
 class HeartbeatSummary(MessageSummary):
-    def render(self):
-        criteria = getUriCriteria(self.parameters)
-        if criteria is not None:
-            ident = env.idmef_db.getHeartbeatIdents(criteria)[0]
-        else:
-            ident = self.parameters["ident"]
+    @view.route("/heartbeats/summary/<analyzerid>:<messageid>")
+    @view.route("/heartbeats/summary/<messageid>")
+    def render(self, analyzerid=None, messageid=None):
+        MessageSummary.render(self)
 
-        heartbeat = env.idmef_db.getHeartbeat(ident, htmlsafe=True)["heartbeat"]
-        self.dataset["sections"] = [ ]
+        heartbeat = env.dataprovider.get(getUriCriteria("heartbeat", analyzerid, messageid))[0]["heartbeat"]
+
+        env.request.dataset["sections"] = [ ]
 
         self.beginSection(_("Heartbeat"))
         self.buildTime(heartbeat)
@@ -1132,4 +1118,3 @@ class HeartbeatSummary(MessageSummary):
         self.endSection()
 
         self.buildAdditionalData(heartbeat)
-
