@@ -1,6 +1,10 @@
-from . import usermanagement
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 from prewikka import pluginmanager, version
 
+from . import usermanagement
+
+
 class UserManagement(pluginmanager.PluginPreload):
     plugin_name = "User management"
     plugin_author = version.__author__
@@ -8,4 +12,4 @@ class UserManagement(pluginmanager.PluginPreload):
     plugin_version = version.__version__
     plugin_copyright = version.__copyright__
     plugin_description = N_("User settings page")
-    plugin_classes = [ usermanagement.UserSettingsDisplay, usermanagement.UserSettingsModify ]
+    plugin_classes = [ usermanagement.UserSettings ]
